@@ -1,6 +1,6 @@
 package Net::Statsd;
 {
-  $Net::Statsd::VERSION = '0.07';
+  $Net::Statsd::VERSION = '0.08';
 }
 
 # ABSTRACT: Sends statistics to the stats daemon over UDP
@@ -14,6 +14,9 @@ use IO::Socket ();
 our $HOST = 'localhost';
 our $PORT = 8125;
 
+my $SOCK;
+my $SOCK_PEER;
+
 
 
 sub timing {
@@ -99,31 +102,42 @@ sub send {
         return;
     }
 
-    my $udp_sock = IO::Socket::INET->new(
-        Proto    => 'udp',
-        PeerAddr => $HOST,
-        PeerPort => $PORT,
-    ) or do {
-        # warn perhaps?
-        return
-    };
+    # cache the socket to avoid dns and socket creation overheads
+    # (this boosts performance from ~6k to >60k sends/sec)
+    if (!$SOCK || !$SOCK_PEER || "$HOST:$PORT" ne $SOCK_PEER) {
 
-    # We don't want to die if Net::Statsd::send() doesn't work...
-    # We could though:
-    #
-    # or die "Could not create UDP socket: $!\n";
+        $SOCK = IO::Socket::INET->new(
+            Proto    => 'udp',
+            PeerAddr => $HOST,
+            PeerPort => $PORT,
+        ) or do {
+            Carp::carp("Net::Statsd can't create a socket to $HOST:$PORT: $!")
+                unless our $_warn_once->{"$HOST:$PORT"}++;
+            return
+        };
+        $SOCK_PEER = "$HOST:$PORT";
+
+        # We don't want to die if Net::Statsd::send() doesn't work...
+        # We could though:
+        #
+        # or die "Could not create UDP socket: $!\n";
+    }
 
     my $all_sent = 1;
 
-    for my $stat (keys %{ $sampled_data }) {
-        my $value = $sampled_data->{$stat};
+    keys %{ $sampled_data }; # reset iterator
+    while ( my ($stat, $value) = each %{ $sampled_data } ) {
         my $packet = "$stat:$value";
-        $udp_sock->send($packet);
-        # XXX If you want warnings...
-        # or do {
-        #    warn "[" . localtime() . "] UDP packet '$packet' send failed\n";
-        #    $all_sent = 0;
-        #};
+        # send() returns the number of characters sent, or undef on error.
+        my $r = send($SOCK, $packet, 0);
+        if (!defined $r) {
+            #warn "Net::Statsd send error: $!";
+            $all_sent = 0;
+        }
+        elsif ($r != length($packet)) {
+            #warn "Net::Statsd send truncated: $!";
+            $all_sent = 0;
+        }
     }
 
     return $all_sent;
@@ -178,7 +192,7 @@ Net::Statsd - Sends statistics to the stats daemon over UDP
 
 =head1 VERSION
 
-version 0.07
+version 0.08
 
 =head1 SYNOPSIS
 
